/* SIS38XXTestMain.cpp */
/* 
 * Nov. 21, 2011
 * Author: Kate Feng, Brookhaven National Laboratory
 * A migration of _APPNAME_Main.cpp from Marty Kraimer 
 * to support RTEMS.
 */

#include <stddef.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "epicsExit.h"
#include "epicsThread.h"
#include "epicsExport.h"
#include "iocsh.h"

#ifdef __rtems__ 
/* There was a bug in RTEMS4.10.0 powerPC port, which restricted the available
 * CPU memory size to be only 32 Mbytes.  I reported it and suggested a patch
 * in May 2011.  The bug is fixed in RTEMS4.10.1.
 * However, instead of using sbrk(heap_sbrk_spared) to extend the remaining
 * memory in RTEMS due to its plan for memory protection , one has to use
 * malloc() to extend it.  There was a bug in EPICSR3.14.12.0
 * that one has to define MALLOC_TEST to extend the available CPU memory size
 * to be beyond 32 Mbytes.  This bug is fixed in EPICSR3.14.12.1 that one 
 * would not have to define MALLOC_TEST.
 *
 * #define MALLOC_TEST
 */

extern "C" {
  extern void save_restoreRegister();
  extern void do_asInitHooksRTEMSRegister();
  extern void do_VME944xRegister();
  extern void do_ScalerRegister();
}

extern int do_motor_registerRecordDeviceDriver();
extern int do_pmc341_registerRecordDeviceDriver();
extern int do_OmsMAXvRegister();
extern int do_Oms58Register();

#ifdef MALLOC_TEST
/* It's really tested for RTEMS 4.9.4 */
#include <rtems/malloc.h>

#define epicsExportSharedSymbols
#include "osiPoolStatus.h"
#endif

extern void drvSIS3820Register(void);

#endif

/* static int done_fw=0;*/

int main(int argc,char *argv[])
{

#ifdef __rtems__ 
   char *ptr, *ptr1;

    save_restoreRegister();
    do_asInitHooksRTEMSRegister();
    do_VME944xRegister();
    do_ScalerRegister();

    /* extend the memory beyond 32M */
#ifdef MALLOC_TEST
    osiSufficentSpaceInPool(0);
    ptr = (char *) malloc(0x2000000);
    free(ptr);
    osiSufficentSpaceInPool(0);
#endif
#endif

    if(argc>=2) {  
      iocsh(argv[1]);
        epicsThreadSleep(.2);
    }
    iocsh(NULL);
    epicsExit(0);
    return(0);
}

#ifdef __rtems__
static int done_motor = do_motor_registerRecordDeviceDriver();
static int done_pmc341 = do_pmc341_registerRecordDeviceDriver();
static int done_oms58 = do_Oms58Register();
static int done_MAXv = do_OmsMAXvRegister();
#endif
